/**
* DevExtreme (ui/overlay.d.ts)
* Version: 21.1.0 (build 21016-0316)
* Build date: Sat Jan 16 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import {
    animationConfig
} from '../animation/fx';

import '../jquery_augmentation';

import {
    dxElement
} from '../core/element';

import {
    template
} from '../core/templates/template';

import {
    event
} from '../events/index';

import Widget, {
    WidgetOptions
} from './widget/ui.widget';

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxOverlayOptions<T = dxOverlay> extends WidgetOptions<T> {
    /**
     */
    animation?: dxOverlayAnimation;
    /**
     */
    closeOnOutsideClick?: boolean | ((event: event) => boolean);
    /**
     */
    contentTemplate?: template | ((contentElement: dxElement) => string | Element | JQuery);
    /**
     */
    deferRendering?: boolean;
    /**
     */
    dragEnabled?: boolean;
    /**
     */
    height?: number | string | (() => number | string);
    /**
     */
    maxHeight?: number | string | (() => number | string);
    /**
     */
    maxWidth?: number | string | (() => number | string);
    /**
     */
    minHeight?: number | string | (() => number | string);
    /**
     */
    minWidth?: number | string | (() => number | string);
    /**
     */
    onHidden?: ((e: { component?: T, element?: dxElement, model?: any }) => any);
    /**
     */
    onHiding?: ((e: { component?: T, element?: dxElement, model?: any, cancel?: boolean }) => any);
    /**
     */
    onShowing?: ((e: { component?: T, element?: dxElement, model?: any }) => any);
    /**
     */
    onShown?: ((e: { component?: T, element?: dxElement, model?: any }) => any);
    /**
     */
    position?: any;
    /**
     */
    shading?: boolean;
    /**
     */
    shadingColor?: string;
    /**
     */
    visible?: boolean;
    /**
     */
    width?: number | string | (() => number | string);
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxOverlayAnimation {
    /**
     * An object that defines the animation properties used when the UI component is being hidden.
     */
    hide?: animationConfig;
    /**
     * An object that defines the animation properties used when the UI component is being shown.
     */
    show?: animationConfig;
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export default class dxOverlay extends Widget {
    constructor(element: Element, options?: dxOverlayOptions)
    constructor(element: JQuery, options?: dxOverlayOptions)
    /**
     */
    content(): dxElement;
    /**
     */
    hide(): Promise<boolean> & JQueryPromise<boolean>;
    /**
     */
    repaint(): void;
    /**
     */
    show(): Promise<boolean> & JQueryPromise<boolean>;
    /**
     */
    toggle(showing: boolean): Promise<boolean> & JQueryPromise<boolean>;
}

/**
 * Specifies the base z-index for all overlay UI components.
 */
export function baseZIndex(zIndex: number): void;

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxOverlayOptions;

/** @deprecated use Options instead */
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxOverlayOptions;
