/**
* DevExtreme (viz/pie_chart.d.ts)
* Version: 21.1.0 (build 21016-0316)
* Build date: Sat Jan 16 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import '../jquery_augmentation';

import {
    dxElement
} from '../core/element';

import {
    template
} from '../core/templates/template';

import {
    event
} from '../events/index';

import {
    format
} from '../ui/widget/ui.widget';

import {
    basePointObject,
    baseSeriesObject
} from './chart';

import {
    PaletteType
} from './palette';

import {
    BaseChart,
    BaseChartAdaptiveLayout,
    BaseChartLegend,
    BaseChartOptions
} from './chart_components/base_chart';

import {
    BaseLegendItem,
    DashStyleType,
    HatchingDirectionType
} from './common';

import {
    Font,
    WordWrapType,
    VizTextOverflowType,
    BaseWidgetAnnotationConfig
} from './core/base_widget';

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export type PieSeriesType = 'donut' | 'doughnut' | 'pie';
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export type SegmentsDirectionType = 'anticlockwise' | 'clockwise';

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface PieChartLegendItem extends BaseLegendItem {
    /**
     */
    argument?: string | Date | number;
    /**
     */
    argumentIndex?: number;
    /**
     */
    points?: Array<piePointObject>;
    /**
     */
    text?: any;
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface PieChartSeries extends dxPieChartSeriesTypesCommonPieChartSeries {
    /**
     */
    name?: string;
    /**
     */
    tag?: any;
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxPieChartOptions extends BaseChartOptions<dxPieChart> {
    /**
     */
    adaptiveLayout?: dxPieChartAdaptiveLayout;
    /**
     */
    centerTemplate?: template | ((component: dxPieChart, element: SVGGElement) => string | SVGElement | JQuery);
    /**
     */
    commonSeriesSettings?: any;
    /**
     */
    diameter?: number;
    /**
     */
    innerRadius?: number;
    /**
     */
    legend?: dxPieChartLegend;
    /**
     */
    minDiameter?: number;
    /**
     */
    onLegendClick?: ((e: { component?: dxPieChart, element?: dxElement, model?: any, event?: event, target?: string | number, points?: Array<piePointObject> }) => any) | string;
    /**
     */
    palette?: Array<string> | PaletteType;
    /**
     */
    resolveLabelOverlapping?: 'hide' | 'none' | 'shift';
    /**
     */
    segmentsDirection?: SegmentsDirectionType;
    /**
     */
    series?: PieChartSeries | Array<PieChartSeries>;
    /**
     */
    seriesTemplate?: {
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      customizeSeries?: ((seriesName: any) => PieChartSeries),
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      nameField?: string
    };
    /**
     */
    sizeGroup?: string;
    /**
     */
    startAngle?: number;
    /**
     */
    type?: PieSeriesType;
    /**
     */
    annotations?: Array<dxPieChartAnnotationConfig | any>;
    /**
     */
    commonAnnotationSettings?: dxPieChartCommonAnnotationConfig;
    /**
     */
    customizeAnnotation?: ((annotation: dxPieChartAnnotationConfig | any) => dxPieChartAnnotationConfig);
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxPieChartAnnotationConfig extends dxPieChartCommonAnnotationConfig {
    /**
     */
    name?: string;
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxPieChartCommonAnnotationConfig extends BaseWidgetAnnotationConfig {
    /**
     */
    location?: 'center' | 'edge';
    /**
     */
    argument?: number | Date | string;
    /**
     */
    series?: string;
    /**
     */
    customizeTooltip?: ((annotation: dxPieChartAnnotationConfig | any) => any);
    /**
     */
    template?: template | ((annotation: dxPieChartAnnotationConfig | any, element: SVGGElement) => string | SVGElement | JQuery);
    /**
     */
    tooltipTemplate?: template | ((annotation: dxPieChartAnnotationConfig | any, element: dxElement) => string | Element | JQuery);
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxPieChartAdaptiveLayout extends BaseChartAdaptiveLayout {
    /**
     * Specifies whether point labels should be kept when the UI component adapts the layout.
     */
    keepLabels?: boolean;
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxPieChartLegend extends BaseChartLegend {
    /**
     * Specifies the text for a hint that appears when a user hovers the mouse pointer over a legend item.
     */
    customizeHint?: ((pointInfo: { pointName?: any, pointIndex?: number, pointColor?: string }) => string);
    /**
     * Allows you to change the order, text, and visibility of legend items.
     */
    customizeItems?: ((items: Array<PieChartLegendItem>) => Array<PieChartLegendItem>);
    /**
     * Specifies a callback function that returns the text to be displayed by a legend item.
     */
    customizeText?: ((pointInfo: { pointName?: any, pointIndex?: number, pointColor?: string }) => string);
    /**
     * Specifies what chart elements to highlight when a corresponding item in the legend is hovered over.
     */
    hoverMode?: 'none' | 'allArgumentPoints';
    /**
     * Specifies an SVG element that serves as a custom legend item marker.
     */
    markerTemplate?: template | ((legendItem: PieChartLegendItem, element: SVGGElement) => string | SVGElement | JQuery);
}
/**
 */
export default class dxPieChart extends BaseChart {
    constructor(element: Element, options?: dxPieChartOptions)
    constructor(element: JQuery, options?: dxPieChartOptions)
    /**
     */
    getInnerRadius(): number;
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxPieChartSeriesTypes {
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
    CommonPieChartSeries?: dxPieChartSeriesTypesCommonPieChartSeries;
    /**
     */
    DoughnutSeries?: any;
    /**
     */
    PieSeries?: any;
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxPieChartSeriesTypesCommonPieChartSeries {
    /**
     * Specifies the data source field that provides arguments for series points.
     */
    argumentField?: string;
    /**
     * Specifies the required type for series arguments.
     */
    argumentType?: 'datetime' | 'numeric' | 'string';
    /**
     * An object defining the series border configuration properties.
     */
    border?: {
      /**
       * Sets a border color for a series.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      color?: string,
      /**
       * Specifies a dash style for the border of a series point.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      dashStyle?: DashStyleType,
      /**
       * Sets border visibility for a series.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      visible?: boolean,
      /**
       * Sets a border width for a series in pixels.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      width?: number
    };
    /**
     * Specifies a series color.
     */
    color?: string;
    /**
     * Specifies the chart elements to highlight when a series is hovered over.
     */
    hoverMode?: 'none' | 'onlyPoint';
    /**
     * An object defining configuration properties for a hovered series.
     */
    hoverStyle?: {
      /**
       * An object defining the border properties for a hovered series.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      border?: {
        /**
         * Sets a border color for the series when it is hovered over.
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        color?: string,
        /**
         * Specifies a dash style for the border of a series point when this point is hovered over.
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        dashStyle?: DashStyleType,
        /**
         * Sets border visibility for a hovered series.
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        visible?: boolean,
        /**
         * Sets a border width for a hovered series.
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        width?: number
      },
      /**
       * Sets a color for the series when it is hovered over.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      color?: string,
      /**
       * Specifies the hatching properties to be applied when a point is hovered over.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      hatching?: {
        /**
         * Specifies how to apply hatching to highlight the hovered point.
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        direction?: HatchingDirectionType,
        /**
         * Specifies the opacity of hatching lines.
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        opacity?: number,
        /**
         * Specifies the distance between two hatching lines in pixels.
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        step?: number,
        /**
         * Specifies the width of hatching lines in pixels.
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        width?: number
      }
    };
    /**
     * An object defining the label configuration properties.
     */
    label?: {
      /**
       * Formats the point argument before it is displayed in the point label. To format the point value, use the format property.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      argumentFormat?: format,
      /**
       * Colors the point labels' background. The default color is inherited from the points.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      backgroundColor?: string,
      /**
       * Specifies border properties for point labels.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      border?: {
        /**
         * Specifies a border color for point labels.
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        color?: string,
        /**
         * Specifies a dash style for the borders of point labels.
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        dashStyle?: DashStyleType,
        /**
         * Indicates whether or not borders are visible in point labels.
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        visible?: boolean,
        /**
         * Specifies the border width for point labels.
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        width?: number
      },
      /**
       * Specifies connector properties for series point labels.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      connector?: {
        /**
         * Specifies the color of label connectors.
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        color?: string,
        /**
         * Indicates whether or not label connectors are visible.
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        visible?: boolean,
        /**
         * Specifies the width of label connectors.
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        width?: number
      },
      /**
       * Specifies a callback function that returns the text to be displayed by point labels.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      customizeText?: ((pointInfo: any) => string),
      /**
       * Specifies font properties for the text displayed in point labels.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      font?: Font,
      /**
       * Formats a value before it is displayed in a point label.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      format?: format,
      /**
       * Specifies a label position relative to the chart.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      position?: 'columns' | 'inside' | 'outside',
      /**
       * Specifies how to shift labels from their initial position in a radial direction in pixels.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      radialOffset?: number,
      /**
       * Specifies the angle used to rotate point labels from their initial position.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      rotationAngle?: number,
      /**
       * Specifies what to do with label texts that overflow the allocated space after applying wordWrap: hide, truncate them and display an ellipsis, or do nothing.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      textOverflow?: VizTextOverflowType,
      /**
       * Specifies the visibility of point labels.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      visible?: boolean,
      /**
       * Specifies how to wrap label texts if they do not fit into a single line.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      wordWrap?: WordWrapType
    };
    /**
     * Specifies how many points are acceptable to be in a series to display all labels for these points. Otherwise, the labels will not be displayed.
     */
    maxLabelCount?: number;
    /**
     * Specifies a minimal size of a displayed pie segment.
     */
    minSegmentSize?: number;
    /**
     * Specifies the chart elements to highlight when the series is selected.
     */
    selectionMode?: 'none' | 'onlyPoint';
    /**
     * An object defining configuration properties for the series when it is selected.
     */
    selectionStyle?: {
      /**
       * An object defining the border properties for a selected series.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      border?: {
        /**
         * Sets a border color for a selected series.
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        color?: string,
        /**
         * Specifies a dash style for the border of a selected series point.
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        dashStyle?: DashStyleType,
        /**
         * Sets a border visibility for a selected series.
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        visible?: boolean,
        /**
         * Sets a border width for a selected series.
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        width?: number
      },
      /**
       * Sets a color for a series when it is selected.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      color?: string,
      /**
       * Specifies the hatching properties to be applied when a point is selected.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      hatching?: {
        /**
         * Specifies how to apply hatching to highlight the selected point.
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        direction?: HatchingDirectionType,
        /**
         * Specifies the opacity of hatching lines.
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        opacity?: number,
        /**
         * Specifies the distance between two hatching lines in pixels.
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        step?: number,
        /**
         * Specifies the width of hatching lines in pixels.
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        width?: number
      }
    };
    /**
     * Specifies chart segment grouping properties.
     */
    smallValuesGrouping?: {
      /**
       * Specifies the name of the grouped chart segment. This name represents the segment in the chart legend.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      groupName?: string,
      /**
       * Specifies the segment grouping mode.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      mode?: 'none' | 'smallValueThreshold' | 'topN',
      /**
       * Specifies a threshold for segment values.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      threshold?: number,
      /**
       * Specifies how many segments must not be grouped.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      topCount?: number
    };
    /**
     * Specifies the name of the data source field that provides data about a point.
     */
    tagField?: string;
    /**
     * Specifies the data source field that provides values for series points.
     */
    valueField?: string;
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface piePointObject extends basePointObject {
    /**
     */
    hide(): void;
    /**
     */
    isVisible(): boolean;
    /**
     */
    percent?: string | number | Date;
    /**
     */
    show(): void;
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface pieChartSeriesObject extends baseSeriesObject {
  /**
   * Warning! This type is used for internal purposes. Do not import it directly.
   */
  hover(): void;
  /**
   * Warning! This type is used for internal purposes. Do not import it directly.
   */
  clearHover(): void;
  /**
   * Warning! This type is used for internal purposes. Do not import it directly.
   */
  isHovered(): boolean;
}

declare global {
interface JQuery {
    dxPieChart(): JQuery;
    dxPieChart(options: "instance"): dxPieChart;
    dxPieChart(options: string): any;
    dxPieChart(options: string, ...params: any[]): any;
    dxPieChart(options: dxPieChartOptions): JQuery;
}
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxPieChartOptions;

/** @deprecated use Options instead */
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxPieChartOptions;
