/**
* DevExtreme (viz/vector_map.d.ts)
* Version: 21.1.0 (build 21016-0316)
* Build date: Sat Jan 16 2021
*
* Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import '../jquery_augmentation';

import {
    dxElement
} from '../core/element';

import {
    PaletteType
} from './palette';

import {
    template
} from '../core/templates/template';

import DataSource, {
    DataSourceOptions
} from '../data/data_source';

import {
    event
} from '../events/index';

import {
    format
} from '../ui/widget/ui.widget';

import {
    BaseLegend,
    BaseLegendItem
} from './common';

import BaseWidget, {
    BaseWidgetMargin,
    BaseWidgetOptions,
    BaseWidgetTooltip,
    Font,
    BaseWidgetAnnotationConfig
} from './core/base_widget';

import {
    VectorMapProjectionConfig
} from './vector_map/projection';

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface MapLayer {
    /**
     */
    clearSelection(): void;
    /**
     */
    elementType?: string;
    /**
     */
    getDataSource(): DataSource;
    /**
     */
    getElements(): Array<MapLayerElement>;
    /**
     */
    index?: number;
    /**
     */
    name?: string;
    /**
     */
    type?: string;
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface MapLayerElement {
    /**
     */
    applySettings(settings: any): void;
    /**
     */
    attribute(name: string): any;
    /**
     */
    attribute(name: string, value: any): void;
    /**
     */
    coordinates(): any;
    /**
     */
    layer?: any;
    /**
     */
    selected(): boolean;
    /**
     */
    selected(state: boolean): void;
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface VectorMapLegendItem extends BaseLegendItem {
    /**
     */
    color?: string;
    /**
     */
    end?: number;
    /**
     */
    size?: number;
    /**
     */
    start?: number;
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxVectorMapOptions extends BaseWidgetOptions<dxVectorMap> {
    /**
     */
    background?: {
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
      borderColor?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      color?: string
    };
    /**
     */
    bounds?: Array<number>;
    /**
     */
    center?: Array<number>;
    /**
     */
    controlBar?: {
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
      borderColor?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      color?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      enabled?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      horizontalAlignment?: 'center' | 'left' | 'right',
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      margin?: number,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      opacity?: number,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      verticalAlignment?: 'bottom' | 'top'
    };
    /**
     */
    layers?: Array<{
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      borderColor?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      borderWidth?: number,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      color?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      colorGroupingField?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      colorGroups?: Array<number>,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      customize?: ((elements: Array<MapLayerElement>) => any),
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      dataField?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      dataSource?: any | DataSource | DataSourceOptions | string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      elementType?: 'bubble' | 'dot' | 'image' | 'pie',
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      hoverEnabled?: boolean,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      hoveredBorderColor?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      hoveredBorderWidth?: number,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      hoveredColor?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      label?: {
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        dataField?: string,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        enabled?: boolean,
        /**
         * Warning! This type is used for internal purposes. Do not import it directly.
         */
        font?: Font
      },
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      maxSize?: number,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      minSize?: number,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      name?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      opacity?: number,
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
      palette?: Array<string> | PaletteType,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      paletteSize?: number,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      selectedBorderColor?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      selectedBorderWidth?: number,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      selectedColor?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      selectionMode?: 'multiple' | 'none' | 'single',
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      size?: number,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      sizeGroupingField?: string,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      sizeGroups?: Array<number>,
      /**
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      type?: 'area' | 'line' | 'marker'
    }> | { borderColor?: string, borderWidth?: number, color?: string, colorGroupingField?: string, colorGroups?: Array<number>, customize?: ((elements: Array<MapLayerElement>) => any), dataField?: string, dataSource?: any | DataSource | DataSourceOptions | string, elementType?: 'bubble' | 'dot' | 'image' | 'pie', hoverEnabled?: boolean, hoveredBorderColor?: string, hoveredBorderWidth?: number, hoveredColor?: string, label?: { dataField?: string, enabled?: boolean, font?: Font }, maxSize?: number, minSize?: number, name?: string, opacity?: number, palette?: Array<string> | PaletteType, paletteSize?: number, selectedBorderColor?: string, selectedBorderWidth?: number, selectedColor?: string, selectionMode?: 'multiple' | 'none' | 'single', size?: number, sizeGroupingField?: string, sizeGroups?: Array<number>, type?: 'area' | 'line' | 'marker' };
    /**
     */
    legends?: Array<dxVectorMapLegends>;
    /**
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
    margin?: BaseWidgetMargin;
    /**
     */
    maxZoomFactor?: number;
    /**
     */
    onCenterChanged?: ((e: { component?: dxVectorMap, element?: dxElement, model?: any, center?: Array<number> }) => any);
    /**
     */
    onClick?: ((e: { component?: dxVectorMap, element?: dxElement, model?: any, event?: event, target?: MapLayerElement }) => any) | string;
    /**
     */
    onSelectionChanged?: ((e: { component?: dxVectorMap, element?: dxElement, model?: any, target?: MapLayerElement }) => any);
    /**
     */
    onTooltipHidden?: ((e: { component?: dxVectorMap, element?: dxElement, model?: any, target?: MapLayerElement | dxVectorMapAnnotationConfig | any }) => any);
    /**
     */
    onTooltipShown?: ((e: { component?: dxVectorMap, element?: dxElement, model?: any, target?: MapLayerElement | dxVectorMapAnnotationConfig | any }) => any);
    /**
     */
    onZoomFactorChanged?: ((e: { component?: dxVectorMap, element?: dxElement, model?: any, zoomFactor?: number }) => any);
    /**
     */
    panningEnabled?: boolean;
    /**
     */
    projection?: 'equirectangular' | 'lambert' | 'mercator' | 'miller' | VectorMapProjectionConfig | string | any;
    /**
     */
    tooltip?: dxVectorMapTooltip;
    /**
     */
    touchEnabled?: boolean;
    /**
     */
    wheelEnabled?: boolean;
    /**
     */
    zoomFactor?: number;
    /**
     */
    zoomingEnabled?: boolean;
    /**
     */
    commonAnnotationSettings?: dxVectorMapCommonAnnotationConfig;
    /**
     */
    annotations?: Array<dxVectorMapAnnotationConfig | any>;
    /**
     */
    customizeAnnotation?: ((annotation: dxVectorMapAnnotationConfig | any) => dxVectorMapAnnotationConfig);
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxVectorMapAnnotationConfig extends dxVectorMapCommonAnnotationConfig {
    /**
     */
    name?: string;
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxVectorMapCommonAnnotationConfig extends BaseWidgetAnnotationConfig {
    /**
     */
    coordinates?: Array<number>;
    /**
     */
    customizeTooltip?: ((annotation: dxVectorMapAnnotationConfig | any) => any);
    /**
     */
    template?: template | ((annotation: dxVectorMapAnnotationConfig | any, element: SVGGElement) => string | SVGElement | JQuery);
    /**
     */
    tooltipTemplate?: template | ((annotation: dxVectorMapAnnotationConfig | any, element: dxElement) => string | Element | JQuery);
}

/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxVectorMapLegends extends BaseLegend {
    /**
     * Specifies text for a hint that appears when a user hovers the mouse pointer over the text of a legend item.
     */
    customizeHint?: ((itemInfo: { start?: number, end?: number, index?: number, color?: string, size?: number }) => string);
    /**
     * Allows you to change the order and visibility of legend items.
     */
    customizeItems?: ((items: Array<VectorMapLegendItem>) => Array<VectorMapLegendItem>);
    /**
     * Specifies text for legend items.
     */
    customizeText?: ((itemInfo: { start?: number, end?: number, index?: number, color?: string, size?: number }) => string);
    /**
     * Specifies the legend items' font properties.
     */
    font?: Font;
    /**
     * Specifies the color of item markers in the legend. The specified color applied only when the legend uses 'size' source.
     */
    markerColor?: string;
    /**
     * Specifies the shape of item markers.
     */
    markerShape?: 'circle' | 'square';
    /**
     * Specifies the marker's size in a legend item in pixels.
     */
    markerSize?: number;
    /**
     * Specifies an SVG element that serves as a custom legend item marker.
     */
    markerTemplate?: template | ((legendItem: VectorMapLegendItem, element: SVGGElement) => string | SVGElement | JQuery);
    /**
     * Specifies the source of data for the legend.
     */
    source?: {
      /**
       * Specifies the type of the legend grouping.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      grouping?: string,
      /**
       * Specifies a layer to which the legend belongs.
       * Warning! This type is used for internal purposes. Do not import it directly.
       */
      layer?: string
    };
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export interface dxVectorMapTooltip extends BaseWidgetTooltip {
    /**
     * Specifies a custom template for a tooltip.
     */
    contentTemplate?: template | ((info: MapLayerElement, element: dxElement) => string | Element | JQuery);
    /**
     * Specifies text and appearance of a set of tooltips.
     */
    customizeTooltip?: ((info: MapLayerElement) => any);
    /**
     * Formats a value before it is displayed it in a tooltip.
     * Warning! This type is used for internal purposes. Do not import it directly.
     */
    format?: format;
}
/**
 */
export default class dxVectorMap extends BaseWidget {
    constructor(element: Element, options?: dxVectorMapOptions)
    constructor(element: JQuery, options?: dxVectorMapOptions)
    /**
     */
    center(): Array<number>;
    /**
     */
    center(centerCoordinates: Array<number>): void;
    /**
     */
    clearSelection(): void;
    /**
     * @deprecated
     */
    convertCoordinates(x: number, y: number): Array<number>;
    /**
     */
    convertToGeo(x: number, y: number): Array<number>;
    /**
     */
    convertToXY(longitude: number, latitude: number): Array<number>;
    /**
     */
    getLayerByIndex(index: number): MapLayer;
    /**
     */
    getLayerByName(name: string): MapLayer;
    /**
     */
    getLayers(): Array<MapLayer>;
    /**
     */
    viewport(): Array<number>;
    /**
     */
    viewport(viewportCoordinates: Array<number>): void;
    /**
     */
    zoomFactor(): number;
    /**
     */
    zoomFactor(zoomFactor: number): void;
}

declare global {
interface JQuery {
    dxVectorMap(): JQuery;
    dxVectorMap(options: "instance"): dxVectorMap;
    dxVectorMap(options: string): any;
    dxVectorMap(options: string, ...params: any[]): any;
    dxVectorMap(options: dxVectorMapOptions): JQuery;
}
}
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export type Options = dxVectorMapOptions;

/** @deprecated use Options instead */
/**
 * Warning! This type is used for internal purposes. Do not import it directly.
 */
export type IOptions = dxVectorMapOptions;
